/****************************************************************************
** Meta object code from reading C++ file 'expressionedit.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/expressionedit.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'expressionedit.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20ExpressionProxyModelE_t {};
} // unnamed namespace

template <> constexpr inline auto ExpressionProxyModel::qt_create_metaobjectdata<qt_meta_tag_ZN20ExpressionProxyModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExpressionProxyModel"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExpressionProxyModel, qt_meta_tag_ZN20ExpressionProxyModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExpressionProxyModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QSortFilterProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20ExpressionProxyModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20ExpressionProxyModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20ExpressionProxyModelE_t>.metaTypes,
    nullptr
} };

void ExpressionProxyModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExpressionProxyModel *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *ExpressionProxyModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExpressionProxyModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20ExpressionProxyModelE_t>.strings))
        return static_cast<void*>(this);
    return QSortFilterProxyModel::qt_metacast(_clname);
}

int ExpressionProxyModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSortFilterProxyModel::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN14ExpressionEditE_t {};
} // unnamed namespace

template <> constexpr inline auto ExpressionEdit::qt_create_metaobjectdata<qt_meta_tag_ZN14ExpressionEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExpressionEdit",
        "returnPressed",
        "",
        "toConversionRequested",
        "std::string",
        "calculateRPNRequest",
        "calculateSelectionRequest",
        "expressionStatusModeChanged",
        "statusChanged",
        "expressionChanged",
        "onTextChanged",
        "onCursorPositionChanged",
        "onCompletionActivated",
        "QModelIndex",
        "onCompletionHighlighted",
        "onCompletionMenuItemActivated",
        "enableIM",
        "enableCompletionDelay",
        "onCompletionModeChanged",
        "onAutocalcSelectionChanged",
        "onStatusModeChanged",
        "onSelectionChanged",
        "showCurrentStatus",
        "updateCompletion",
        "setExpression",
        "str",
        "blockCompletion",
        "blockParseStatus",
        "blockUndo",
        "hideCompletion",
        "addToHistory",
        "smartParentheses",
        "insertBrackets",
        "selectAll",
        "b",
        "editUndo",
        "editRedo",
        "editDelete",
        "insertDate",
        "insertMatrix",
        "enableTabCompletion",
        "completeOrActivateFirst",
        "complete",
        "MathStructure*",
        "QMenu*",
        "clearHistory",
        "updateDigitGroups"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'returnPressed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'toConversionRequested'
        QtMocHelpers::SignalData<void(std::string)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Signal 'calculateRPNRequest'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'calculateSelectionRequest'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'expressionStatusModeChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(QString, bool, bool, bool, bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::Bool, 2 }, { QMetaType::Bool, 2 }, { QMetaType::Bool, 2 },
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'expressionChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onTextChanged'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onTextChanged'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onCursorPositionChanged'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCompletionActivated'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Slot 'onCompletionHighlighted'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 2 },
        }}),
        // Slot 'onCompletionMenuItemActivated'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'enableIM'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'enableCompletionDelay'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCompletionModeChanged'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onAutocalcSelectionChanged'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onStatusModeChanged'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onSelectionChanged'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'showCurrentStatus'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateCompletion'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setExpression'
        QtMocHelpers::SlotData<void(std::string)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Slot 'setExpression'
        QtMocHelpers::SlotData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Slot 'blockCompletion'
        QtMocHelpers::SlotData<void(bool, bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Slot 'blockCompletion'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'blockCompletion'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'blockParseStatus'
        QtMocHelpers::SlotData<void(bool)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'blockParseStatus'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'blockUndo'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'blockUndo'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'hideCompletion'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addToHistory'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'smartParentheses'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertBrackets'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 34 },
        }}),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'editUndo'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editRedo'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editDelete'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertDate'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertMatrix'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'enableTabCompletion'
        QtMocHelpers::SlotData<void(bool)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'completeOrActivateFirst'
        QtMocHelpers::SlotData<bool(bool)>(41, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'completeOrActivateFirst'
        QtMocHelpers::SlotData<bool()>(41, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'complete'
        QtMocHelpers::SlotData<bool(MathStructure *, MathStructure *, QMenu *, bool, bool)>(42, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 43, 2 }, { 0x80000000 | 43, 2 }, { 0x80000000 | 44, 2 }, { QMetaType::Bool, 2 },
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'complete'
        QtMocHelpers::SlotData<bool(MathStructure *, MathStructure *, QMenu *, bool)>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 43, 2 }, { 0x80000000 | 43, 2 }, { 0x80000000 | 44, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Slot 'complete'
        QtMocHelpers::SlotData<bool(MathStructure *, MathStructure *, QMenu *)>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 43, 2 }, { 0x80000000 | 43, 2 }, { 0x80000000 | 44, 2 },
        }}),
        // Slot 'complete'
        QtMocHelpers::SlotData<bool(MathStructure *, MathStructure *)>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 43, 2 }, { 0x80000000 | 43, 2 },
        }}),
        // Slot 'complete'
        QtMocHelpers::SlotData<bool(MathStructure *)>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 43, 2 },
        }}),
        // Slot 'complete'
        QtMocHelpers::SlotData<bool()>(42, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'clearHistory'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateDigitGroups'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExpressionEdit, qt_meta_tag_ZN14ExpressionEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExpressionEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<QPlainTextEdit::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExpressionEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExpressionEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14ExpressionEditE_t>.metaTypes,
    nullptr
} };

void ExpressionEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExpressionEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->returnPressed(); break;
        case 1: _t->toConversionRequested((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1]))); break;
        case 2: _t->calculateRPNRequest((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->calculateSelectionRequest(); break;
        case 4: _t->expressionStatusModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 6: _t->expressionChanged(); break;
        case 7: _t->onTextChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->onTextChanged(); break;
        case 9: _t->onCursorPositionChanged(); break;
        case 10: _t->onCompletionActivated((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 11: _t->onCompletionHighlighted((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 12: _t->onCompletionMenuItemActivated(); break;
        case 13: _t->enableIM(); break;
        case 14: _t->enableCompletionDelay(); break;
        case 15: _t->onCompletionModeChanged(); break;
        case 16: _t->onAutocalcSelectionChanged(); break;
        case 17: _t->onStatusModeChanged(); break;
        case 18: _t->onSelectionChanged(); break;
        case 19: _t->showCurrentStatus(); break;
        case 20: _t->updateCompletion(); break;
        case 21: _t->setExpression((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1]))); break;
        case 22: _t->setExpression((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 23: _t->blockCompletion((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 24: _t->blockCompletion((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 25: _t->blockCompletion(); break;
        case 26: _t->blockParseStatus((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->blockParseStatus(); break;
        case 28: _t->blockUndo((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->blockUndo(); break;
        case 30: _t->hideCompletion(); break;
        case 31: _t->addToHistory(); break;
        case 32: _t->smartParentheses(); break;
        case 33: _t->insertBrackets(); break;
        case 34: _t->selectAll((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->selectAll(); break;
        case 36: _t->editUndo(); break;
        case 37: _t->editRedo(); break;
        case 38: _t->editDelete(); break;
        case 39: _t->insertDate(); break;
        case 40: _t->insertMatrix(); break;
        case 41: _t->enableTabCompletion((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 42: { bool _r = _t->completeOrActivateFirst((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 43: { bool _r = _t->completeOrActivateFirst();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 44: { bool _r = _t->complete((*reinterpret_cast<std::add_pointer_t<MathStructure*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MathStructure*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMenu*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 45: { bool _r = _t->complete((*reinterpret_cast<std::add_pointer_t<MathStructure*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MathStructure*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMenu*>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 46: { bool _r = _t->complete((*reinterpret_cast<std::add_pointer_t<MathStructure*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MathStructure*>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QMenu*>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 47: { bool _r = _t->complete((*reinterpret_cast<std::add_pointer_t<MathStructure*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<MathStructure*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 48: { bool _r = _t->complete((*reinterpret_cast<std::add_pointer_t<MathStructure*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 49: { bool _r = _t->complete();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 50: _t->clearHistory(); break;
        case 51: _t->updateDigitGroups(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ExpressionEdit::*)()>(_a, &ExpressionEdit::returnPressed, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExpressionEdit::*)(std::string )>(_a, &ExpressionEdit::toConversionRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExpressionEdit::*)(int )>(_a, &ExpressionEdit::calculateRPNRequest, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExpressionEdit::*)()>(_a, &ExpressionEdit::calculateSelectionRequest, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExpressionEdit::*)(bool )>(_a, &ExpressionEdit::expressionStatusModeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExpressionEdit::*)(QString , bool , bool , bool , bool )>(_a, &ExpressionEdit::statusChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExpressionEdit::*)()>(_a, &ExpressionEdit::expressionChanged, 6))
            return;
    }
}

const QMetaObject *ExpressionEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExpressionEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14ExpressionEditE_t>.strings))
        return static_cast<void*>(this);
    return QPlainTextEdit::qt_metacast(_clname);
}

int ExpressionEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPlainTextEdit::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 52)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 52;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 52)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 52;
    }
    return _id;
}

// SIGNAL 0
void ExpressionEdit::returnPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ExpressionEdit::toConversionRequested(std::string _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ExpressionEdit::calculateRPNRequest(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ExpressionEdit::calculateSelectionRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ExpressionEdit::expressionStatusModeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void ExpressionEdit::statusChanged(QString _t1, bool _t2, bool _t3, bool _t4, bool _t5)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1, _t2, _t3, _t4, _t5);
}

// SIGNAL 6
void ExpressionEdit::expressionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
