/****************************************************************************
** Meta object code from reading C++ file 'keypadwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/keypadwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'keypadwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12KeypadButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto KeypadButton::qt_create_metaobjectdata<qt_meta_tag_ZN12KeypadButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KeypadButton",
        "clicked2",
        "",
        "clicked3",
        "longPressTimeout",
        "menuOpened",
        "menuClosed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clicked2'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clicked3'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'longPressTimeout'
        QtMocHelpers::SlotData<void()>(4, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'menuOpened'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'menuClosed'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KeypadButton, qt_meta_tag_ZN12KeypadButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KeypadButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KeypadButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KeypadButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KeypadButtonE_t>.metaTypes,
    nullptr
} };

void KeypadButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KeypadButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clicked2(); break;
        case 1: _t->clicked3(); break;
        case 2: _t->longPressTimeout(); break;
        case 3: _t->menuOpened(); break;
        case 4: _t->menuClosed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KeypadButton::*)()>(_a, &KeypadButton::clicked2, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadButton::*)()>(_a, &KeypadButton::clicked3, 1))
            return;
    }
}

const QMetaObject *KeypadButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KeypadButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KeypadButtonE_t>.strings))
        return static_cast<void*>(this);
    return QToolButton::qt_metacast(_clname);
}

int KeypadButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void KeypadButton::clicked2()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void KeypadButton::clicked3()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
namespace {
struct qt_meta_tag_ZN12KeypadWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto KeypadWidget::qt_create_metaobjectdata<qt_meta_tag_ZN12KeypadWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KeypadWidget",
        "operatorClicked",
        "",
        "symbolClicked",
        "functionClicked",
        "MathFunction*",
        "f",
        "variableClicked",
        "Variable*",
        "v",
        "unitClicked",
        "Unit*",
        "u",
        "prefixClicked",
        "Prefix*",
        "p",
        "delClicked",
        "clearClicked",
        "equalsClicked",
        "parenthesesClicked",
        "bracketsClicked",
        "leftClicked",
        "rightClicked",
        "endClicked",
        "startClicked",
        "MSClicked",
        "MCClicked",
        "MRClicked",
        "MPlusClicked",
        "MMinusClicked",
        "storeClicked",
        "newFunctionClicked",
        "backspaceClicked",
        "answerClicked",
        "baseClicked",
        "factorizeClicked",
        "expandClicked",
        "expandPartialFractionsClicked",
        "expressionCalculationUpdated",
        "expressionFormatUpdated",
        "resultFormatUpdated",
        "shortcutClicked",
        "openVariablesRequest",
        "openUnitsRequest",
        "openPercentageCalculationRequest",
        "onSymbolButtonClicked",
        "onOperatorButtonClicked",
        "onItemButtonClicked",
        "onUnitItemClicked",
        "onPrefixItemClicked",
        "onUnitButtonClicked2",
        "onUnitButtonClicked3",
        "onSymbolButtonClicked2",
        "onOperatorButtonClicked2",
        "onItemButtonClicked2",
        "onSymbolButtonClicked3",
        "onOperatorButtonClicked3",
        "onItemButtonClicked3",
        "onBaseButtonClicked",
        "onBaseButtonClicked2",
        "onCustomButtonClicked",
        "onCustomButtonClicked2",
        "onCustomButtonClicked3",
        "onCustomButton2Clicked",
        "onCustomButton2Clicked2",
        "onCustomButton2Clicked3",
        "onHypToggled",
        "assumptionsTypeActivated",
        "assumptionsSignActivated",
        "defaultAssumptionsActivated",
        "updateAssumptions",
        "onCustomEditClicked",
        "onCustomEdit2Clicked",
        "removeCustomRow",
        "addCustomRow",
        "removeCustomColumn",
        "addCustomColumn",
        "updateCustomActionOK",
        "customActionOKClicked",
        "currentCustomActionChanged",
        "QTreeWidgetItem*",
        "intervalDisplayActivated",
        "intervalCalculationActivated",
        "conciseInputActivated",
        "variableUnitsActivated",
        "updateIntervalMenu",
        "complexFormActivated",
        "updateComplexMenu",
        "updateStoreMenu",
        "updateSinMenu",
        "updateCosMenu",
        "updateTanMenu",
        "updatePiMenu",
        "updateLnMenu",
        "updateSqrtMenu",
        "updatePowerMenu",
        "updateEMenu",
        "updateFactorialMenu",
        "updateSumMenu",
        "updateXMenu",
        "updateEqualsMenu",
        "updateFactorizeMenu",
        "updatePercentageMenu",
        "updateUnitsMenu",
        "updateBase",
        "updateSymbols",
        "setKeypadType",
        "hideNumpad",
        "showCustomButtonColumn",
        "showSeparateKeypadMenuButtons",
        "updateButtonLocation"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'operatorClicked'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'symbolClicked'
        QtMocHelpers::SignalData<void(const QString &)>(3, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Signal 'functionClicked'
        QtMocHelpers::SignalData<void(MathFunction *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'variableClicked'
        QtMocHelpers::SignalData<void(Variable *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'unitClicked'
        QtMocHelpers::SignalData<void(Unit *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Signal 'prefixClicked'
        QtMocHelpers::SignalData<void(Prefix *)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 15 },
        }}),
        // Signal 'delClicked'
        QtMocHelpers::SignalData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'clearClicked'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'equalsClicked'
        QtMocHelpers::SignalData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'parenthesesClicked'
        QtMocHelpers::SignalData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'bracketsClicked'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'leftClicked'
        QtMocHelpers::SignalData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rightClicked'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'endClicked'
        QtMocHelpers::SignalData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startClicked'
        QtMocHelpers::SignalData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'MSClicked'
        QtMocHelpers::SignalData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'MCClicked'
        QtMocHelpers::SignalData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'MRClicked'
        QtMocHelpers::SignalData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'MPlusClicked'
        QtMocHelpers::SignalData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'MMinusClicked'
        QtMocHelpers::SignalData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'storeClicked'
        QtMocHelpers::SignalData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'newFunctionClicked'
        QtMocHelpers::SignalData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'backspaceClicked'
        QtMocHelpers::SignalData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'answerClicked'
        QtMocHelpers::SignalData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'baseClicked'
        QtMocHelpers::SignalData<void(int, bool, bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Bool, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Signal 'factorizeClicked'
        QtMocHelpers::SignalData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'expandClicked'
        QtMocHelpers::SignalData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'expandPartialFractionsClicked'
        QtMocHelpers::SignalData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'expressionCalculationUpdated'
        QtMocHelpers::SignalData<void(int)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'expressionFormatUpdated'
        QtMocHelpers::SignalData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Signal 'resultFormatUpdated'
        QtMocHelpers::SignalData<void(int)>(40, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Signal 'shortcutClicked'
        QtMocHelpers::SignalData<void(int, const QString &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Signal 'openVariablesRequest'
        QtMocHelpers::SignalData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openUnitsRequest'
        QtMocHelpers::SignalData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'openPercentageCalculationRequest'
        QtMocHelpers::SignalData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onSymbolButtonClicked'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onOperatorButtonClicked'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onItemButtonClicked'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onUnitItemClicked'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onPrefixItemClicked'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onUnitButtonClicked2'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onUnitButtonClicked3'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onSymbolButtonClicked2'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onOperatorButtonClicked2'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onItemButtonClicked2'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onSymbolButtonClicked3'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onOperatorButtonClicked3'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onItemButtonClicked3'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onBaseButtonClicked'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onBaseButtonClicked2'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCustomButtonClicked'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCustomButtonClicked2'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCustomButtonClicked3'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCustomButton2Clicked'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCustomButton2Clicked2'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCustomButton2Clicked3'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onHypToggled'
        QtMocHelpers::SlotData<void(bool)>(66, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'assumptionsTypeActivated'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'assumptionsSignActivated'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'defaultAssumptionsActivated'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateAssumptions'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCustomEditClicked'
        QtMocHelpers::SlotData<void(bool)>(71, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onCustomEdit2Clicked'
        QtMocHelpers::SlotData<void(bool)>(72, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'removeCustomRow'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addCustomRow'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'removeCustomColumn'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addCustomColumn'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateCustomActionOK'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'customActionOKClicked'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'currentCustomActionChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(79, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 80, 2 }, { 0x80000000 | 80, 2 },
        }}),
        // Slot 'intervalDisplayActivated'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'intervalCalculationActivated'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'conciseInputActivated'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'variableUnitsActivated'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateIntervalMenu'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'complexFormActivated'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateComplexMenu'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateStoreMenu'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateSinMenu'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateCosMenu'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateTanMenu'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updatePiMenu'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateLnMenu'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateSqrtMenu'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updatePowerMenu'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateEMenu'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateFactorialMenu'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateSumMenu'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateXMenu'
        QtMocHelpers::SlotData<void()>(99, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateEqualsMenu'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateFactorizeMenu'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updatePercentageMenu'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateUnitsMenu'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateBase'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateSymbols'
        QtMocHelpers::SlotData<void()>(105, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setKeypadType'
        QtMocHelpers::SlotData<void(int)>(106, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'hideNumpad'
        QtMocHelpers::SlotData<void(bool)>(107, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'showCustomButtonColumn'
        QtMocHelpers::SlotData<void(bool)>(108, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'showSeparateKeypadMenuButtons'
        QtMocHelpers::SlotData<void(bool)>(109, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'updateButtonLocation'
        QtMocHelpers::SlotData<void()>(110, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KeypadWidget, qt_meta_tag_ZN12KeypadWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KeypadWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KeypadWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KeypadWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12KeypadWidgetE_t>.metaTypes,
    nullptr
} };

void KeypadWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KeypadWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->operatorClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->symbolClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->functionClicked((*reinterpret_cast<std::add_pointer_t<MathFunction*>>(_a[1]))); break;
        case 3: _t->variableClicked((*reinterpret_cast<std::add_pointer_t<Variable*>>(_a[1]))); break;
        case 4: _t->unitClicked((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 5: _t->prefixClicked((*reinterpret_cast<std::add_pointer_t<Prefix*>>(_a[1]))); break;
        case 6: _t->delClicked(); break;
        case 7: _t->clearClicked(); break;
        case 8: _t->equalsClicked(); break;
        case 9: _t->parenthesesClicked(); break;
        case 10: _t->bracketsClicked(); break;
        case 11: _t->leftClicked(); break;
        case 12: _t->rightClicked(); break;
        case 13: _t->endClicked(); break;
        case 14: _t->startClicked(); break;
        case 15: _t->MSClicked(); break;
        case 16: _t->MCClicked(); break;
        case 17: _t->MRClicked(); break;
        case 18: _t->MPlusClicked(); break;
        case 19: _t->MMinusClicked(); break;
        case 20: _t->storeClicked(); break;
        case 21: _t->newFunctionClicked(); break;
        case 22: _t->backspaceClicked(); break;
        case 23: _t->answerClicked(); break;
        case 24: _t->baseClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 25: _t->factorizeClicked(); break;
        case 26: _t->expandClicked(); break;
        case 27: _t->expandPartialFractionsClicked(); break;
        case 28: _t->expressionCalculationUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 29: _t->expressionFormatUpdated((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->resultFormatUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 31: _t->shortcutClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 32: _t->openVariablesRequest(); break;
        case 33: _t->openUnitsRequest(); break;
        case 34: _t->openPercentageCalculationRequest(); break;
        case 35: _t->onSymbolButtonClicked(); break;
        case 36: _t->onOperatorButtonClicked(); break;
        case 37: _t->onItemButtonClicked(); break;
        case 38: _t->onUnitItemClicked(); break;
        case 39: _t->onPrefixItemClicked(); break;
        case 40: _t->onUnitButtonClicked2(); break;
        case 41: _t->onUnitButtonClicked3(); break;
        case 42: _t->onSymbolButtonClicked2(); break;
        case 43: _t->onOperatorButtonClicked2(); break;
        case 44: _t->onItemButtonClicked2(); break;
        case 45: _t->onSymbolButtonClicked3(); break;
        case 46: _t->onOperatorButtonClicked3(); break;
        case 47: _t->onItemButtonClicked3(); break;
        case 48: _t->onBaseButtonClicked(); break;
        case 49: _t->onBaseButtonClicked2(); break;
        case 50: _t->onCustomButtonClicked(); break;
        case 51: _t->onCustomButtonClicked2(); break;
        case 52: _t->onCustomButtonClicked3(); break;
        case 53: _t->onCustomButton2Clicked(); break;
        case 54: _t->onCustomButton2Clicked2(); break;
        case 55: _t->onCustomButton2Clicked3(); break;
        case 56: _t->onHypToggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 57: _t->assumptionsTypeActivated(); break;
        case 58: _t->assumptionsSignActivated(); break;
        case 59: _t->defaultAssumptionsActivated(); break;
        case 60: _t->updateAssumptions(); break;
        case 61: _t->onCustomEditClicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 62: _t->onCustomEdit2Clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 63: _t->removeCustomRow(); break;
        case 64: _t->addCustomRow(); break;
        case 65: _t->removeCustomColumn(); break;
        case 66: _t->addCustomColumn(); break;
        case 67: _t->updateCustomActionOK(); break;
        case 68: _t->customActionOKClicked(); break;
        case 69: _t->currentCustomActionChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 70: _t->intervalDisplayActivated(); break;
        case 71: _t->intervalCalculationActivated(); break;
        case 72: _t->conciseInputActivated(); break;
        case 73: _t->variableUnitsActivated(); break;
        case 74: _t->updateIntervalMenu(); break;
        case 75: _t->complexFormActivated(); break;
        case 76: _t->updateComplexMenu(); break;
        case 77: _t->updateStoreMenu(); break;
        case 78: _t->updateSinMenu(); break;
        case 79: _t->updateCosMenu(); break;
        case 80: _t->updateTanMenu(); break;
        case 81: _t->updatePiMenu(); break;
        case 82: _t->updateLnMenu(); break;
        case 83: _t->updateSqrtMenu(); break;
        case 84: _t->updatePowerMenu(); break;
        case 85: _t->updateEMenu(); break;
        case 86: _t->updateFactorialMenu(); break;
        case 87: _t->updateSumMenu(); break;
        case 88: _t->updateXMenu(); break;
        case 89: _t->updateEqualsMenu(); break;
        case 90: _t->updateFactorizeMenu(); break;
        case 91: _t->updatePercentageMenu(); break;
        case 92: _t->updateUnitsMenu(); break;
        case 93: _t->updateBase(); break;
        case 94: _t->updateSymbols(); break;
        case 95: _t->setKeypadType((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 96: _t->hideNumpad((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 97: _t->showCustomButtonColumn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 98: _t->showSeparateKeypadMenuButtons((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 99: _t->updateButtonLocation(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(const QString & )>(_a, &KeypadWidget::operatorClicked, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(const QString & )>(_a, &KeypadWidget::symbolClicked, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(MathFunction * )>(_a, &KeypadWidget::functionClicked, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(Variable * )>(_a, &KeypadWidget::variableClicked, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(Unit * )>(_a, &KeypadWidget::unitClicked, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(Prefix * )>(_a, &KeypadWidget::prefixClicked, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::delClicked, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::clearClicked, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::equalsClicked, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::parenthesesClicked, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::bracketsClicked, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::leftClicked, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::rightClicked, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::endClicked, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::startClicked, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::MSClicked, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::MCClicked, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::MRClicked, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::MPlusClicked, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::MMinusClicked, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::storeClicked, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::newFunctionClicked, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::backspaceClicked, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::answerClicked, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(int , bool , bool )>(_a, &KeypadWidget::baseClicked, 24))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::factorizeClicked, 25))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::expandClicked, 26))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::expandPartialFractionsClicked, 27))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(int )>(_a, &KeypadWidget::expressionCalculationUpdated, 28))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(bool )>(_a, &KeypadWidget::expressionFormatUpdated, 29))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(int )>(_a, &KeypadWidget::resultFormatUpdated, 30))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)(int , const QString & )>(_a, &KeypadWidget::shortcutClicked, 31))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::openVariablesRequest, 32))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::openUnitsRequest, 33))
            return;
        if (QtMocHelpers::indexOfMethod<void (KeypadWidget::*)()>(_a, &KeypadWidget::openPercentageCalculationRequest, 34))
            return;
    }
}

const QMetaObject *KeypadWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KeypadWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12KeypadWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int KeypadWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 100)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 100;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 100)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 100;
    }
    return _id;
}

// SIGNAL 0
void KeypadWidget::operatorClicked(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KeypadWidget::symbolClicked(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void KeypadWidget::functionClicked(MathFunction * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void KeypadWidget::variableClicked(Variable * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void KeypadWidget::unitClicked(Unit * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void KeypadWidget::prefixClicked(Prefix * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void KeypadWidget::delClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void KeypadWidget::clearClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void KeypadWidget::equalsClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void KeypadWidget::parenthesesClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void KeypadWidget::bracketsClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void KeypadWidget::leftClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void KeypadWidget::rightClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 12, nullptr);
}

// SIGNAL 13
void KeypadWidget::endClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 13, nullptr);
}

// SIGNAL 14
void KeypadWidget::startClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void KeypadWidget::MSClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 15, nullptr);
}

// SIGNAL 16
void KeypadWidget::MCClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 16, nullptr);
}

// SIGNAL 17
void KeypadWidget::MRClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void KeypadWidget::MPlusClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 18, nullptr);
}

// SIGNAL 19
void KeypadWidget::MMinusClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 19, nullptr);
}

// SIGNAL 20
void KeypadWidget::storeClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void KeypadWidget::newFunctionClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 21, nullptr);
}

// SIGNAL 22
void KeypadWidget::backspaceClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 22, nullptr);
}

// SIGNAL 23
void KeypadWidget::answerClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 23, nullptr);
}

// SIGNAL 24
void KeypadWidget::baseClicked(int _t1, bool _t2, bool _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 24, nullptr, _t1, _t2, _t3);
}

// SIGNAL 25
void KeypadWidget::factorizeClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 25, nullptr);
}

// SIGNAL 26
void KeypadWidget::expandClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 26, nullptr);
}

// SIGNAL 27
void KeypadWidget::expandPartialFractionsClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 27, nullptr);
}

// SIGNAL 28
void KeypadWidget::expressionCalculationUpdated(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 28, nullptr, _t1);
}

// SIGNAL 29
void KeypadWidget::expressionFormatUpdated(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 29, nullptr, _t1);
}

// SIGNAL 30
void KeypadWidget::resultFormatUpdated(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 30, nullptr, _t1);
}

// SIGNAL 31
void KeypadWidget::shortcutClicked(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 31, nullptr, _t1, _t2);
}

// SIGNAL 32
void KeypadWidget::openVariablesRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 32, nullptr);
}

// SIGNAL 33
void KeypadWidget::openUnitsRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 33, nullptr);
}

// SIGNAL 34
void KeypadWidget::openPercentageCalculationRequest()
{
    QMetaObject::activate(this, &staticMetaObject, 34, nullptr);
}
QT_WARNING_POP
