/****************************************************************************
** Meta object code from reading C++ file 'variablesdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/variablesdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'variablesdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15VariablesDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto VariablesDialog::qt_create_metaobjectdata<qt_meta_tag_ZN15VariablesDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "VariablesDialog",
        "itemsChanged",
        "",
        "favouritesChanged",
        "unitRemoved",
        "Unit*",
        "unitDeactivated",
        "applyVariableRequest",
        "Variable*",
        "insertVariableRequest",
        "selectedCategoryChanged",
        "QTreeWidgetItem*",
        "selectedVariableChanged",
        "QModelIndex",
        "newVariable",
        "newMatrix",
        "newUnknown",
        "editClicked",
        "delClicked",
        "insertClicked",
        "exportClicked",
        "deactivateClicked",
        "favouriteClicked",
        "searchChanged",
        "reject"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'itemsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'favouritesChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'unitRemoved'
        QtMocHelpers::SignalData<void(Unit *)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'unitDeactivated'
        QtMocHelpers::SignalData<void(Unit *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 2 },
        }}),
        // Signal 'applyVariableRequest'
        QtMocHelpers::SignalData<void(Variable *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Signal 'insertVariableRequest'
        QtMocHelpers::SignalData<void(Variable *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 2 },
        }}),
        // Slot 'selectedCategoryChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(10, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 11, 2 }, { 0x80000000 | 11, 2 },
        }}),
        // Slot 'selectedVariableChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(12, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 13, 2 }, { 0x80000000 | 13, 2 },
        }}),
        // Slot 'newVariable'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'newMatrix'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'newUnknown'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editClicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'delClicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertClicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'exportClicked'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'deactivateClicked'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'favouriteClicked'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'searchChanged'
        QtMocHelpers::SlotData<void(const QString &)>(23, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'reject'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<VariablesDialog, qt_meta_tag_ZN15VariablesDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject VariablesDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15VariablesDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15VariablesDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15VariablesDialogE_t>.metaTypes,
    nullptr
} };

void VariablesDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<VariablesDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->itemsChanged(); break;
        case 1: _t->favouritesChanged(); break;
        case 2: _t->unitRemoved((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 3: _t->unitDeactivated((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 4: _t->applyVariableRequest((*reinterpret_cast<std::add_pointer_t<Variable*>>(_a[1]))); break;
        case 5: _t->insertVariableRequest((*reinterpret_cast<std::add_pointer_t<Variable*>>(_a[1]))); break;
        case 6: _t->selectedCategoryChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 7: _t->selectedVariableChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 8: _t->newVariable(); break;
        case 9: _t->newMatrix(); break;
        case 10: _t->newUnknown(); break;
        case 11: _t->editClicked(); break;
        case 12: _t->delClicked(); break;
        case 13: _t->insertClicked(); break;
        case 14: _t->exportClicked(); break;
        case 15: _t->deactivateClicked(); break;
        case 16: _t->favouriteClicked(); break;
        case 17: _t->searchChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 18: _t->reject(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (VariablesDialog::*)()>(_a, &VariablesDialog::itemsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (VariablesDialog::*)()>(_a, &VariablesDialog::favouritesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (VariablesDialog::*)(Unit * )>(_a, &VariablesDialog::unitRemoved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (VariablesDialog::*)(Unit * )>(_a, &VariablesDialog::unitDeactivated, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (VariablesDialog::*)(Variable * )>(_a, &VariablesDialog::applyVariableRequest, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (VariablesDialog::*)(Variable * )>(_a, &VariablesDialog::insertVariableRequest, 5))
            return;
    }
}

const QMetaObject *VariablesDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *VariablesDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15VariablesDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int VariablesDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void VariablesDialog::itemsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void VariablesDialog::favouritesChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void VariablesDialog::unitRemoved(Unit * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void VariablesDialog::unitDeactivated(Unit * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void VariablesDialog::applyVariableRequest(Variable * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void VariablesDialog::insertVariableRequest(Variable * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
