/****************************************************************************
** Meta object code from reading C++ file 'qalculatewindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../src/qalculatewindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qalculatewindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QalculateWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto QalculateWindow::qt_create_metaobjectdata<qt_meta_tag_ZN15QalculateWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QalculateWindow",
        "testTimeout",
        "",
        "updateBinEditSize",
        "QFont*",
        "onBinaryBitsChanged",
        "onTwosChanged",
        "onSymbolClicked",
        "onOperatorClicked",
        "onFunctionClicked",
        "MathFunction*",
        "onVariableClicked",
        "Variable*",
        "onUnitClicked",
        "Unit*",
        "onPrefixClicked",
        "Prefix*",
        "onClearClicked",
        "onDelClicked",
        "onEqualsClicked",
        "onParenthesesClicked",
        "onBracketsClicked",
        "onBackspaceClicked",
        "onLeftClicked",
        "onRightClicked",
        "onStartClicked",
        "onEndClicked",
        "onMSClicked",
        "onMRClicked",
        "onMCClicked",
        "onMPlusClicked",
        "onMMinusClicked",
        "onAnswerClicked",
        "onBaseClicked",
        "onFactorizeClicked",
        "onExpandClicked",
        "onExpandPartialFractionsClicked",
        "onToActivated",
        "onToActivatedAlt",
        "onStoreActivated",
        "keypadTypeActivated",
        "showNumpad",
        "showCustomKeypadColumn",
        "showSeparateKeypadMenuButtons",
        "resetKeypadPosition",
        "onEMHTimeout",
        "onResizeTimeout",
        "onKeypadVisibilityChanged",
        "onToolbarVisibilityChanged",
        "onBasesActivated",
        "onBasesVisibilityChanged",
        "onRPNVisibilityChanged",
        "onRPNClosed",
        "onExpressionChanged",
        "onHistoryReloaded",
        "onStatusChanged",
        "autoCalculateTimeout",
        "onToConversionRequested",
        "std::string",
        "onInsertTextRequested",
        "onInsertValueRequested",
        "onAlwaysOnTopChanged",
        "onEnableTooltipsChanged",
        "onPreferencesClosed",
        "onTitleTypeChanged",
        "onResultFontChanged",
        "onExpressionFontChanged",
        "onStatusFontChanged",
        "onKeypadFontChanged",
        "onAppFontChanged",
        "onAppFontTimer",
        "fixSplitterPos",
        "angleUnitActivated",
        "normalActivated",
        "scientificActivated",
        "engineeringActivated",
        "simpleActivated",
        "onPrecisionChanged",
        "syncDecimals",
        "onMinDecimalsChanged",
        "onMaxDecimalsChanged",
        "outputBaseActivated",
        "onCustomOutputBaseChanged",
        "inputBaseActivated",
        "onCustomInputBaseChanged",
        "assumptionsTypeActivated",
        "assumptionsSignActivated",
        "approximationActivated",
        "openCalendarConversion",
        "onInsertFunctionExec",
        "onInsertFunctionInsertRPN",
        "onInsertFunctionKeepOpen",
        "onInsertFunctionClosed",
        "onInsertFunctionChanged",
        "onInsertFunctionEntryActivated",
        "insertFunctionDo",
        "FunctionDialog*",
        "onEntrySelectFile",
        "onEntryEditMatrix",
        "onCalculateFunctionRequested",
        "onInsertFunctionRequested",
        "onUnitActivated",
        "u",
        "onUnitRemoved",
        "onVariableRemoved",
        "onUnitDeactivated",
        "onVariableDeactivated",
        "normalModeActivated",
        "rpnModeActivated",
        "chainModeActivated",
        "registerUp",
        "registerDown",
        "registerSwap",
        "copyRegister",
        "rpnLastX",
        "deleteRegister",
        "clearStack",
        "registerChanged",
        "calculateRPN",
        "approximateResult",
        "toggleResultFraction",
        "onExpressionStatusModeChanged",
        "functionActivated",
        "unitActivated",
        "prefixActivated",
        "variableActivated",
        "updateAngleUnitsMenu",
        "initializeFunctionsMenu",
        "initializeVariablesMenu",
        "initializeUnitsMenu",
        "updateFunctionsMenu",
        "useFunctionDialog",
        "showAllFunctions",
        "updateFavouriteFunctions",
        "updateRecentFunctions",
        "addToRecentFunctions",
        "updateUnitsMenu",
        "addCurrencyFlagsToMenu",
        "showAllUnits",
        "updateFavouriteUnits",
        "updateRecentUnits",
        "addToRecentUnits",
        "updateVariablesMenu",
        "showAllVariables",
        "updateFavouriteVariables",
        "updateRecentVariables",
        "addToRecentVariables",
        "shortcutActivated",
        "shortcutClicked",
        "keyboardShortcutAdded",
        "keyboard_shortcut*",
        "ks",
        "keyboardShortcutRemoved",
        "addShortcutClicked",
        "editShortcutClicked",
        "removeShortcutClicked",
        "shortcutActionOKClicked",
        "shortcutActionAddClicked",
        "updateShortcutActionOK",
        "currentShortcutActionChanged",
        "currentShortcutChanged",
        "QTreeWidgetItem*",
        "shortcutDoubleClicked",
        "saveWorkspace",
        "saveWorkspaceAs",
        "openWorkspace",
        "openDefaultWorkspace",
        "openRecentWorkspace",
        "onColorSchemeChanged",
        "showToolbarContextMenu",
        "QPoint",
        "setToolbarStyle",
        "showKeypadContextMenu",
        "updateKeypadTitle",
        "keypadPreferencesChanged",
        "resultBasesLinkActivated",
        "showBasesContextMenu",
        "copyBases",
        "onExpressionPositionChanged",
        "onShowStatusBarChanged",
        "serverNewConnection",
        "socketReadyRead",
        "calculate",
        "calculateSelection",
        "onActivateRequested",
        "abort",
        "abortCommand",
        "fetchExchangeRates",
        "showAbout",
        "expressionCalculationUpdated",
        "delay",
        "resultFormatUpdated",
        "resultDisplayUpdated",
        "expressionObjectsUpdated",
        "expressionFormatUpdated",
        "insertFunction",
        "QWidget*",
        "newVariable",
        "newUnit",
        "newMatrix",
        "newUnknown",
        "newFunction",
        "convertToUnit",
        "importCSV",
        "exportCSV",
        "editKeyboardShortcuts",
        "editPreferences",
        "openSettingsFolder",
        "onDatasetsChanged",
        "onFunctionsChanged",
        "insertProperty",
        "DataObject*",
        "DataProperty*",
        "openDatasets",
        "openFunctions",
        "resetUnitsAndVariablesMenus",
        "openUnits",
        "openVariables",
        "applyFunction",
        "openFPConversion",
        "openPercentageCalculation",
        "openPlot",
        "openPeriodicTable",
        "negate",
        "checkVersion",
        "reportBug",
        "help",
        "loadInitialHistory",
        "startTest"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'testTimeout'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateBinEditSize'
        QtMocHelpers::SlotData<void(QFont *)>(3, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 4, 2 },
        }}),
        // Slot 'updateBinEditSize'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onBinaryBitsChanged'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onTwosChanged'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onSymbolClicked'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'onOperatorClicked'
        QtMocHelpers::SlotData<void(const QString &)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'onFunctionClicked'
        QtMocHelpers::SlotData<void(MathFunction *)>(9, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'onVariableClicked'
        QtMocHelpers::SlotData<void(Variable *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'onUnitClicked'
        QtMocHelpers::SlotData<void(Unit *)>(13, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'onPrefixClicked'
        QtMocHelpers::SlotData<void(Prefix *)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 16, 2 },
        }}),
        // Slot 'onClearClicked'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onDelClicked'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onEqualsClicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onParenthesesClicked'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onBracketsClicked'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onBackspaceClicked'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onLeftClicked'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onRightClicked'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onStartClicked'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onEndClicked'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onMSClicked'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onMRClicked'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onMCClicked'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onMPlusClicked'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onMMinusClicked'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onAnswerClicked'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onBaseClicked'
        QtMocHelpers::SlotData<void(int, bool, bool)>(33, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::Bool, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Slot 'onFactorizeClicked'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onExpandClicked'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onExpandPartialFractionsClicked'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onToActivated'
        QtMocHelpers::SlotData<void(bool)>(37, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onToActivated'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'onToActivatedAlt'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onStoreActivated'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'keypadTypeActivated'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'showNumpad'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'showCustomKeypadColumn'
        QtMocHelpers::SlotData<void(bool)>(42, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'showSeparateKeypadMenuButtons'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'resetKeypadPosition'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onEMHTimeout'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onResizeTimeout'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onKeypadVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onToolbarVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(48, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onBasesActivated'
        QtMocHelpers::SlotData<void(bool)>(49, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onBasesVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(50, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onRPNVisibilityChanged'
        QtMocHelpers::SlotData<void(bool)>(51, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onRPNClosed'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onExpressionChanged'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onHistoryReloaded'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onStatusChanged'
        QtMocHelpers::SlotData<void(QString, bool, bool, bool, bool)>(55, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 }, { QMetaType::Bool, 2 }, { QMetaType::Bool, 2 }, { QMetaType::Bool, 2 },
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'autoCalculateTimeout'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onToConversionRequested'
        QtMocHelpers::SlotData<void(std::string)>(57, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 58, 2 },
        }}),
        // Slot 'onInsertTextRequested'
        QtMocHelpers::SlotData<void(std::string)>(59, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 58, 2 },
        }}),
        // Slot 'onInsertValueRequested'
        QtMocHelpers::SlotData<void(int)>(60, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'onAlwaysOnTopChanged'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onEnableTooltipsChanged'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onPreferencesClosed'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onTitleTypeChanged'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onResultFontChanged'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onExpressionFontChanged'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onStatusFontChanged'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onKeypadFontChanged'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onAppFontChanged'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onAppFontTimer'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'fixSplitterPos'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'angleUnitActivated'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'normalActivated'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'scientificActivated'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'engineeringActivated'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'simpleActivated'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onPrecisionChanged'
        QtMocHelpers::SlotData<void(int)>(77, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'syncDecimals'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onMinDecimalsChanged'
        QtMocHelpers::SlotData<void(int)>(79, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'onMaxDecimalsChanged'
        QtMocHelpers::SlotData<void(int)>(80, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'outputBaseActivated'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCustomOutputBaseChanged'
        QtMocHelpers::SlotData<void(int)>(82, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'inputBaseActivated'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCustomInputBaseChanged'
        QtMocHelpers::SlotData<void(int)>(84, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'assumptionsTypeActivated'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'assumptionsSignActivated'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'approximationActivated'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'openCalendarConversion'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onInsertFunctionExec'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onInsertFunctionInsertRPN'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onInsertFunctionKeepOpen'
        QtMocHelpers::SlotData<void(bool)>(91, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onInsertFunctionClosed'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onInsertFunctionChanged'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onInsertFunctionEntryActivated'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'insertFunctionDo'
        QtMocHelpers::SlotData<void(FunctionDialog *)>(95, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 96, 2 },
        }}),
        // Slot 'onEntrySelectFile'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onEntryEditMatrix'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onCalculateFunctionRequested'
        QtMocHelpers::SlotData<void(MathFunction *)>(99, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'onInsertFunctionRequested'
        QtMocHelpers::SlotData<void(MathFunction *)>(100, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'onUnitActivated'
        QtMocHelpers::SlotData<void(Unit *)>(101, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 102 },
        }}),
        // Slot 'onUnitRemoved'
        QtMocHelpers::SlotData<void(Unit *)>(103, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'onVariableRemoved'
        QtMocHelpers::SlotData<void(Variable *)>(104, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'onUnitDeactivated'
        QtMocHelpers::SlotData<void(Unit *)>(105, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'onVariableDeactivated'
        QtMocHelpers::SlotData<void(Variable *)>(106, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'normalModeActivated'
        QtMocHelpers::SlotData<void()>(107, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'rpnModeActivated'
        QtMocHelpers::SlotData<void()>(108, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'chainModeActivated'
        QtMocHelpers::SlotData<void()>(109, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'registerUp'
        QtMocHelpers::SlotData<void()>(110, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'registerDown'
        QtMocHelpers::SlotData<void()>(111, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'registerSwap'
        QtMocHelpers::SlotData<void()>(112, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'copyRegister'
        QtMocHelpers::SlotData<void()>(113, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'rpnLastX'
        QtMocHelpers::SlotData<void()>(114, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'deleteRegister'
        QtMocHelpers::SlotData<void()>(115, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'clearStack'
        QtMocHelpers::SlotData<void()>(116, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'registerChanged'
        QtMocHelpers::SlotData<void(int)>(117, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'calculateRPN'
        QtMocHelpers::SlotData<void(int)>(118, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'approximateResult'
        QtMocHelpers::SlotData<void()>(119, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'toggleResultFraction'
        QtMocHelpers::SlotData<void()>(120, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onExpressionStatusModeChanged'
        QtMocHelpers::SlotData<void(bool)>(121, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'onExpressionStatusModeChanged'
        QtMocHelpers::SlotData<void()>(121, 2, QMC::AccessProtected | QMC::MethodCloned, QMetaType::Void),
        // Slot 'functionActivated'
        QtMocHelpers::SlotData<void()>(122, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'unitActivated'
        QtMocHelpers::SlotData<void()>(123, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'prefixActivated'
        QtMocHelpers::SlotData<void()>(124, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'variableActivated'
        QtMocHelpers::SlotData<void()>(125, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateAngleUnitsMenu'
        QtMocHelpers::SlotData<void()>(126, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'initializeFunctionsMenu'
        QtMocHelpers::SlotData<void()>(127, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'initializeVariablesMenu'
        QtMocHelpers::SlotData<void()>(128, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'initializeUnitsMenu'
        QtMocHelpers::SlotData<void()>(129, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateFunctionsMenu'
        QtMocHelpers::SlotData<void()>(130, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'useFunctionDialog'
        QtMocHelpers::SlotData<void(bool)>(131, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'showAllFunctions'
        QtMocHelpers::SlotData<void(bool)>(132, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'updateFavouriteFunctions'
        QtMocHelpers::SlotData<void()>(133, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateRecentFunctions'
        QtMocHelpers::SlotData<void()>(134, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addToRecentFunctions'
        QtMocHelpers::SlotData<void(MathFunction *)>(135, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'updateUnitsMenu'
        QtMocHelpers::SlotData<void()>(136, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addCurrencyFlagsToMenu'
        QtMocHelpers::SlotData<void()>(137, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'showAllUnits'
        QtMocHelpers::SlotData<void(bool)>(138, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'updateFavouriteUnits'
        QtMocHelpers::SlotData<void()>(139, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateRecentUnits'
        QtMocHelpers::SlotData<void()>(140, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addToRecentUnits'
        QtMocHelpers::SlotData<void(Unit *)>(141, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'updateVariablesMenu'
        QtMocHelpers::SlotData<void()>(142, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'showAllVariables'
        QtMocHelpers::SlotData<void(bool)>(143, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'updateFavouriteVariables'
        QtMocHelpers::SlotData<void()>(144, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateRecentVariables'
        QtMocHelpers::SlotData<void()>(145, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'addToRecentVariables'
        QtMocHelpers::SlotData<void(Variable *)>(146, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 2 },
        }}),
        // Slot 'shortcutActivated'
        QtMocHelpers::SlotData<void()>(147, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'shortcutClicked'
        QtMocHelpers::SlotData<void(int, const QString &)>(148, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot 'keyboardShortcutAdded'
        QtMocHelpers::SlotData<void(keyboard_shortcut *)>(149, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 150, 151 },
        }}),
        // Slot 'keyboardShortcutRemoved'
        QtMocHelpers::SlotData<void(keyboard_shortcut *)>(152, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 150, 151 },
        }}),
        // Slot 'addShortcutClicked'
        QtMocHelpers::SlotData<void()>(153, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'editShortcutClicked'
        QtMocHelpers::SlotData<void()>(154, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'removeShortcutClicked'
        QtMocHelpers::SlotData<void()>(155, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'shortcutActionOKClicked'
        QtMocHelpers::SlotData<void()>(156, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'shortcutActionAddClicked'
        QtMocHelpers::SlotData<void()>(157, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateShortcutActionOK'
        QtMocHelpers::SlotData<void()>(158, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'currentShortcutActionChanged'
        QtMocHelpers::SlotData<void()>(159, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'currentShortcutChanged'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, QTreeWidgetItem *)>(160, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 161, 2 }, { 0x80000000 | 161, 2 },
        }}),
        // Slot 'shortcutDoubleClicked'
        QtMocHelpers::SlotData<void(QTreeWidgetItem *, int)>(162, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 161, 2 }, { QMetaType::Int, 2 },
        }}),
        // Slot 'saveWorkspace'
        QtMocHelpers::SlotData<void()>(163, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'saveWorkspaceAs'
        QtMocHelpers::SlotData<void()>(164, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'openWorkspace'
        QtMocHelpers::SlotData<void()>(165, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'openDefaultWorkspace'
        QtMocHelpers::SlotData<void()>(166, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'openRecentWorkspace'
        QtMocHelpers::SlotData<void()>(167, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onColorSchemeChanged'
        QtMocHelpers::SlotData<void()>(168, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'showToolbarContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(169, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 170, 2 },
        }}),
        // Slot 'setToolbarStyle'
        QtMocHelpers::SlotData<void()>(171, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'showKeypadContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(172, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 170, 2 },
        }}),
        // Slot 'updateKeypadTitle'
        QtMocHelpers::SlotData<void()>(173, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'keypadPreferencesChanged'
        QtMocHelpers::SlotData<void()>(174, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'resultBasesLinkActivated'
        QtMocHelpers::SlotData<void(const QString &)>(175, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'showBasesContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(176, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 170, 2 },
        }}),
        // Slot 'copyBases'
        QtMocHelpers::SlotData<void()>(177, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onExpressionPositionChanged'
        QtMocHelpers::SlotData<void()>(178, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'onShowStatusBarChanged'
        QtMocHelpers::SlotData<void()>(179, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'serverNewConnection'
        QtMocHelpers::SlotData<void()>(180, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'socketReadyRead'
        QtMocHelpers::SlotData<void()>(181, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'calculate'
        QtMocHelpers::SlotData<void()>(182, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'calculate'
        QtMocHelpers::SlotData<void(const QString &)>(182, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'calculateSelection'
        QtMocHelpers::SlotData<void()>(183, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onActivateRequested'
        QtMocHelpers::SlotData<void(const QStringList &, const QString &)>(184, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 2 }, { QMetaType::QString, 2 },
        }}),
        // Slot 'abort'
        QtMocHelpers::SlotData<void()>(185, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'abortCommand'
        QtMocHelpers::SlotData<void()>(186, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'fetchExchangeRates'
        QtMocHelpers::SlotData<void()>(187, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showAbout'
        QtMocHelpers::SlotData<void()>(188, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'expressionCalculationUpdated'
        QtMocHelpers::SlotData<void(int)>(189, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 190 },
        }}),
        // Slot 'expressionCalculationUpdated'
        QtMocHelpers::SlotData<void()>(189, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'resultFormatUpdated'
        QtMocHelpers::SlotData<void(int)>(191, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 190 },
        }}),
        // Slot 'resultFormatUpdated'
        QtMocHelpers::SlotData<void()>(191, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'resultDisplayUpdated'
        QtMocHelpers::SlotData<void()>(192, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'expressionObjectsUpdated'
        QtMocHelpers::SlotData<void()>(193, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'expressionFormatUpdated'
        QtMocHelpers::SlotData<void(bool)>(194, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'insertFunction'
        QtMocHelpers::SlotData<void(MathFunction *, QWidget *)>(195, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 }, { 0x80000000 | 196, 2 },
        }}),
        // Slot 'insertFunction'
        QtMocHelpers::SlotData<void(MathFunction *)>(195, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'newVariable'
        QtMocHelpers::SlotData<void()>(197, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newUnit'
        QtMocHelpers::SlotData<void()>(198, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newMatrix'
        QtMocHelpers::SlotData<void()>(199, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newUnknown'
        QtMocHelpers::SlotData<void()>(200, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'newFunction'
        QtMocHelpers::SlotData<void()>(201, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'convertToUnit'
        QtMocHelpers::SlotData<void(Unit *)>(202, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 14, 2 },
        }}),
        // Slot 'importCSV'
        QtMocHelpers::SlotData<void()>(203, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'exportCSV'
        QtMocHelpers::SlotData<void()>(204, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editKeyboardShortcuts'
        QtMocHelpers::SlotData<void()>(205, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'editPreferences'
        QtMocHelpers::SlotData<void()>(206, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openSettingsFolder'
        QtMocHelpers::SlotData<void()>(207, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onDatasetsChanged'
        QtMocHelpers::SlotData<void()>(208, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onFunctionsChanged'
        QtMocHelpers::SlotData<void()>(209, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'insertProperty'
        QtMocHelpers::SlotData<void(DataObject *, DataProperty *)>(210, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 211, 2 }, { 0x80000000 | 212, 2 },
        }}),
        // Slot 'openDatasets'
        QtMocHelpers::SlotData<void()>(213, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openFunctions'
        QtMocHelpers::SlotData<void()>(214, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'resetUnitsAndVariablesMenus'
        QtMocHelpers::SlotData<void()>(215, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openUnits'
        QtMocHelpers::SlotData<void()>(216, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openVariables'
        QtMocHelpers::SlotData<void()>(217, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'applyFunction'
        QtMocHelpers::SlotData<void(MathFunction *)>(218, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'openFPConversion'
        QtMocHelpers::SlotData<void()>(219, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openPercentageCalculation'
        QtMocHelpers::SlotData<void()>(220, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openPlot'
        QtMocHelpers::SlotData<void()>(221, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openPeriodicTable'
        QtMocHelpers::SlotData<void()>(222, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'negate'
        QtMocHelpers::SlotData<void()>(223, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'checkVersion'
        QtMocHelpers::SlotData<void()>(224, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'reportBug'
        QtMocHelpers::SlotData<void()>(225, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'help'
        QtMocHelpers::SlotData<void()>(226, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'loadInitialHistory'
        QtMocHelpers::SlotData<void()>(227, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startTest'
        QtMocHelpers::SlotData<void()>(228, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QalculateWindow, qt_meta_tag_ZN15QalculateWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QalculateWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QalculateWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QalculateWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QalculateWindowE_t>.metaTypes,
    nullptr
} };

void QalculateWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QalculateWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->testTimeout(); break;
        case 1: _t->updateBinEditSize((*reinterpret_cast<std::add_pointer_t<QFont*>>(_a[1]))); break;
        case 2: _t->updateBinEditSize(); break;
        case 3: _t->onBinaryBitsChanged(); break;
        case 4: _t->onTwosChanged(); break;
        case 5: _t->onSymbolClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->onOperatorClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->onFunctionClicked((*reinterpret_cast<std::add_pointer_t<MathFunction*>>(_a[1]))); break;
        case 8: _t->onVariableClicked((*reinterpret_cast<std::add_pointer_t<Variable*>>(_a[1]))); break;
        case 9: _t->onUnitClicked((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 10: _t->onPrefixClicked((*reinterpret_cast<std::add_pointer_t<Prefix*>>(_a[1]))); break;
        case 11: _t->onClearClicked(); break;
        case 12: _t->onDelClicked(); break;
        case 13: _t->onEqualsClicked(); break;
        case 14: _t->onParenthesesClicked(); break;
        case 15: _t->onBracketsClicked(); break;
        case 16: _t->onBackspaceClicked(); break;
        case 17: _t->onLeftClicked(); break;
        case 18: _t->onRightClicked(); break;
        case 19: _t->onStartClicked(); break;
        case 20: _t->onEndClicked(); break;
        case 21: _t->onMSClicked(); break;
        case 22: _t->onMRClicked(); break;
        case 23: _t->onMCClicked(); break;
        case 24: _t->onMPlusClicked(); break;
        case 25: _t->onMMinusClicked(); break;
        case 26: _t->onAnswerClicked(); break;
        case 27: _t->onBaseClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 28: _t->onFactorizeClicked(); break;
        case 29: _t->onExpandClicked(); break;
        case 30: _t->onExpandPartialFractionsClicked(); break;
        case 31: _t->onToActivated((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 32: _t->onToActivated(); break;
        case 33: _t->onToActivatedAlt(); break;
        case 34: _t->onStoreActivated(); break;
        case 35: _t->keypadTypeActivated(); break;
        case 36: _t->showNumpad((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->showCustomKeypadColumn((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 38: _t->showSeparateKeypadMenuButtons((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 39: _t->resetKeypadPosition(); break;
        case 40: _t->onEMHTimeout(); break;
        case 41: _t->onResizeTimeout(); break;
        case 42: _t->onKeypadVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 43: _t->onToolbarVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 44: _t->onBasesActivated((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 45: _t->onBasesVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->onRPNVisibilityChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 47: _t->onRPNClosed(); break;
        case 48: _t->onExpressionChanged(); break;
        case 49: _t->onHistoryReloaded(); break;
        case 50: _t->onStatusChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[5]))); break;
        case 51: _t->autoCalculateTimeout(); break;
        case 52: _t->onToConversionRequested((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1]))); break;
        case 53: _t->onInsertTextRequested((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1]))); break;
        case 54: _t->onInsertValueRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 55: _t->onAlwaysOnTopChanged(); break;
        case 56: _t->onEnableTooltipsChanged(); break;
        case 57: _t->onPreferencesClosed(); break;
        case 58: _t->onTitleTypeChanged(); break;
        case 59: _t->onResultFontChanged(); break;
        case 60: _t->onExpressionFontChanged(); break;
        case 61: _t->onStatusFontChanged(); break;
        case 62: _t->onKeypadFontChanged(); break;
        case 63: _t->onAppFontChanged(); break;
        case 64: _t->onAppFontTimer(); break;
        case 65: _t->fixSplitterPos(); break;
        case 66: _t->angleUnitActivated(); break;
        case 67: _t->normalActivated(); break;
        case 68: _t->scientificActivated(); break;
        case 69: _t->engineeringActivated(); break;
        case 70: _t->simpleActivated(); break;
        case 71: _t->onPrecisionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 72: _t->syncDecimals(); break;
        case 73: _t->onMinDecimalsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 74: _t->onMaxDecimalsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 75: _t->outputBaseActivated(); break;
        case 76: _t->onCustomOutputBaseChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 77: _t->inputBaseActivated(); break;
        case 78: _t->onCustomInputBaseChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 79: _t->assumptionsTypeActivated(); break;
        case 80: _t->assumptionsSignActivated(); break;
        case 81: _t->approximationActivated(); break;
        case 82: _t->openCalendarConversion(); break;
        case 83: _t->onInsertFunctionExec(); break;
        case 84: _t->onInsertFunctionInsertRPN(); break;
        case 85: _t->onInsertFunctionKeepOpen((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 86: _t->onInsertFunctionClosed(); break;
        case 87: _t->onInsertFunctionChanged(); break;
        case 88: _t->onInsertFunctionEntryActivated(); break;
        case 89: _t->insertFunctionDo((*reinterpret_cast<std::add_pointer_t<FunctionDialog*>>(_a[1]))); break;
        case 90: _t->onEntrySelectFile(); break;
        case 91: _t->onEntryEditMatrix(); break;
        case 92: _t->onCalculateFunctionRequested((*reinterpret_cast<std::add_pointer_t<MathFunction*>>(_a[1]))); break;
        case 93: _t->onInsertFunctionRequested((*reinterpret_cast<std::add_pointer_t<MathFunction*>>(_a[1]))); break;
        case 94: _t->onUnitActivated((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 95: _t->onUnitRemoved((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 96: _t->onVariableRemoved((*reinterpret_cast<std::add_pointer_t<Variable*>>(_a[1]))); break;
        case 97: _t->onUnitDeactivated((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 98: _t->onVariableDeactivated((*reinterpret_cast<std::add_pointer_t<Variable*>>(_a[1]))); break;
        case 99: _t->normalModeActivated(); break;
        case 100: _t->rpnModeActivated(); break;
        case 101: _t->chainModeActivated(); break;
        case 102: _t->registerUp(); break;
        case 103: _t->registerDown(); break;
        case 104: _t->registerSwap(); break;
        case 105: _t->copyRegister(); break;
        case 106: _t->rpnLastX(); break;
        case 107: _t->deleteRegister(); break;
        case 108: _t->clearStack(); break;
        case 109: _t->registerChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 110: _t->calculateRPN((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 111: _t->approximateResult(); break;
        case 112: _t->toggleResultFraction(); break;
        case 113: _t->onExpressionStatusModeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 114: _t->onExpressionStatusModeChanged(); break;
        case 115: _t->functionActivated(); break;
        case 116: _t->unitActivated(); break;
        case 117: _t->prefixActivated(); break;
        case 118: _t->variableActivated(); break;
        case 119: _t->updateAngleUnitsMenu(); break;
        case 120: _t->initializeFunctionsMenu(); break;
        case 121: _t->initializeVariablesMenu(); break;
        case 122: _t->initializeUnitsMenu(); break;
        case 123: _t->updateFunctionsMenu(); break;
        case 124: _t->useFunctionDialog((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 125: _t->showAllFunctions((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 126: _t->updateFavouriteFunctions(); break;
        case 127: _t->updateRecentFunctions(); break;
        case 128: _t->addToRecentFunctions((*reinterpret_cast<std::add_pointer_t<MathFunction*>>(_a[1]))); break;
        case 129: _t->updateUnitsMenu(); break;
        case 130: _t->addCurrencyFlagsToMenu(); break;
        case 131: _t->showAllUnits((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 132: _t->updateFavouriteUnits(); break;
        case 133: _t->updateRecentUnits(); break;
        case 134: _t->addToRecentUnits((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 135: _t->updateVariablesMenu(); break;
        case 136: _t->showAllVariables((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 137: _t->updateFavouriteVariables(); break;
        case 138: _t->updateRecentVariables(); break;
        case 139: _t->addToRecentVariables((*reinterpret_cast<std::add_pointer_t<Variable*>>(_a[1]))); break;
        case 140: _t->shortcutActivated(); break;
        case 141: _t->shortcutClicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 142: _t->keyboardShortcutAdded((*reinterpret_cast<std::add_pointer_t<keyboard_shortcut*>>(_a[1]))); break;
        case 143: _t->keyboardShortcutRemoved((*reinterpret_cast<std::add_pointer_t<keyboard_shortcut*>>(_a[1]))); break;
        case 144: _t->addShortcutClicked(); break;
        case 145: _t->editShortcutClicked(); break;
        case 146: _t->removeShortcutClicked(); break;
        case 147: _t->shortcutActionOKClicked(); break;
        case 148: _t->shortcutActionAddClicked(); break;
        case 149: _t->updateShortcutActionOK(); break;
        case 150: _t->currentShortcutActionChanged(); break;
        case 151: _t->currentShortcutChanged((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[2]))); break;
        case 152: _t->shortcutDoubleClicked((*reinterpret_cast<std::add_pointer_t<QTreeWidgetItem*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 153: _t->saveWorkspace(); break;
        case 154: _t->saveWorkspaceAs(); break;
        case 155: _t->openWorkspace(); break;
        case 156: _t->openDefaultWorkspace(); break;
        case 157: _t->openRecentWorkspace(); break;
        case 158: _t->onColorSchemeChanged(); break;
        case 159: _t->showToolbarContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 160: _t->setToolbarStyle(); break;
        case 161: _t->showKeypadContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 162: _t->updateKeypadTitle(); break;
        case 163: _t->keypadPreferencesChanged(); break;
        case 164: _t->resultBasesLinkActivated((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 165: _t->showBasesContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 166: _t->copyBases(); break;
        case 167: _t->onExpressionPositionChanged(); break;
        case 168: _t->onShowStatusBarChanged(); break;
        case 169: _t->serverNewConnection(); break;
        case 170: _t->socketReadyRead(); break;
        case 171: _t->calculate(); break;
        case 172: _t->calculate((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 173: _t->calculateSelection(); break;
        case 174: _t->onActivateRequested((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 175: _t->abort(); break;
        case 176: _t->abortCommand(); break;
        case 177: _t->fetchExchangeRates(); break;
        case 178: _t->showAbout(); break;
        case 179: _t->expressionCalculationUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 180: _t->expressionCalculationUpdated(); break;
        case 181: _t->resultFormatUpdated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 182: _t->resultFormatUpdated(); break;
        case 183: _t->resultDisplayUpdated(); break;
        case 184: _t->expressionObjectsUpdated(); break;
        case 185: _t->expressionFormatUpdated((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 186: _t->insertFunction((*reinterpret_cast<std::add_pointer_t<MathFunction*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[2]))); break;
        case 187: _t->insertFunction((*reinterpret_cast<std::add_pointer_t<MathFunction*>>(_a[1]))); break;
        case 188: _t->newVariable(); break;
        case 189: _t->newUnit(); break;
        case 190: _t->newMatrix(); break;
        case 191: _t->newUnknown(); break;
        case 192: _t->newFunction(); break;
        case 193: _t->convertToUnit((*reinterpret_cast<std::add_pointer_t<Unit*>>(_a[1]))); break;
        case 194: _t->importCSV(); break;
        case 195: _t->exportCSV(); break;
        case 196: _t->editKeyboardShortcuts(); break;
        case 197: _t->editPreferences(); break;
        case 198: _t->openSettingsFolder(); break;
        case 199: _t->onDatasetsChanged(); break;
        case 200: _t->onFunctionsChanged(); break;
        case 201: _t->insertProperty((*reinterpret_cast<std::add_pointer_t<DataObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<DataProperty*>>(_a[2]))); break;
        case 202: _t->openDatasets(); break;
        case 203: _t->openFunctions(); break;
        case 204: _t->resetUnitsAndVariablesMenus(); break;
        case 205: _t->openUnits(); break;
        case 206: _t->openVariables(); break;
        case 207: _t->applyFunction((*reinterpret_cast<std::add_pointer_t<MathFunction*>>(_a[1]))); break;
        case 208: _t->openFPConversion(); break;
        case 209: _t->openPercentageCalculation(); break;
        case 210: _t->openPlot(); break;
        case 211: _t->openPeriodicTable(); break;
        case 212: _t->negate(); break;
        case 213: _t->checkVersion(); break;
        case 214: _t->reportBug(); break;
        case 215: _t->help(); break;
        case 216: _t->loadInitialHistory(); break;
        case 217: _t->startTest(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 186:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
}

const QMetaObject *QalculateWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QalculateWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QalculateWindowE_t>.strings))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int QalculateWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 218)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 218;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 218)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 218;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN19QalculateTranslatorE_t {};
} // unnamed namespace

template <> constexpr inline auto QalculateTranslator::qt_create_metaobjectdata<qt_meta_tag_ZN19QalculateTranslatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QalculateTranslator"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QalculateTranslator, qt_meta_tag_ZN19QalculateTranslatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QalculateTranslator::staticMetaObject = { {
    QMetaObject::SuperData::link<QTranslator::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QalculateTranslatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QalculateTranslatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QalculateTranslatorE_t>.metaTypes,
    nullptr
} };

void QalculateTranslator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QalculateTranslator *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *QalculateTranslator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QalculateTranslator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QalculateTranslatorE_t>.strings))
        return static_cast<void*>(this);
    return QTranslator::qt_metacast(_clname);
}

int QalculateTranslator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTranslator::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN11MathSpinBoxE_t {};
} // unnamed namespace

template <> constexpr inline auto MathSpinBox::qt_create_metaobjectdata<qt_meta_tag_ZN11MathSpinBoxE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MathSpinBox",
        "returnPressed",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'returnPressed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MathSpinBox, qt_meta_tag_ZN11MathSpinBoxE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MathSpinBox::staticMetaObject = { {
    QMetaObject::SuperData::link<QSpinBox::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MathSpinBoxE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MathSpinBoxE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11MathSpinBoxE_t>.metaTypes,
    nullptr
} };

void MathSpinBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MathSpinBox *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->returnPressed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MathSpinBox::*)()>(_a, &MathSpinBox::returnPressed, 0))
            return;
    }
}

const QMetaObject *MathSpinBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MathSpinBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11MathSpinBoxE_t>.strings))
        return static_cast<void*>(this);
    return QSpinBox::qt_metacast(_clname);
}

int MathSpinBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSpinBox::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void MathSpinBox::returnPressed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN19QalculateDockWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto QalculateDockWidget::qt_create_metaobjectdata<qt_meta_tag_ZN19QalculateDockWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QalculateDockWidget",
        "dockClosed",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'dockClosed'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QalculateDockWidget, qt_meta_tag_ZN19QalculateDockWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QalculateDockWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QDockWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QalculateDockWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QalculateDockWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QalculateDockWidgetE_t>.metaTypes,
    nullptr
} };

void QalculateDockWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QalculateDockWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->dockClosed(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QalculateDockWidget::*)()>(_a, &QalculateDockWidget::dockClosed, 0))
            return;
    }
}

const QMetaObject *QalculateDockWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QalculateDockWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QalculateDockWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QDockWidget::qt_metacast(_clname);
}

int QalculateDockWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDockWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QalculateDockWidget::dockClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
namespace {
struct qt_meta_tag_ZN19QalculateToolButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto QalculateToolButton::qt_create_metaobjectdata<qt_meta_tag_ZN19QalculateToolButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QalculateToolButton",
        "middleButtonClicked",
        "",
        "longPressTimeout"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'middleButtonClicked'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'longPressTimeout'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QalculateToolButton, qt_meta_tag_ZN19QalculateToolButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QalculateToolButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QToolButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QalculateToolButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QalculateToolButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19QalculateToolButtonE_t>.metaTypes,
    nullptr
} };

void QalculateToolButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QalculateToolButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->middleButtonClicked(); break;
        case 1: _t->longPressTimeout(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QalculateToolButton::*)()>(_a, &QalculateToolButton::middleButtonClicked, 0))
            return;
    }
}

const QMetaObject *QalculateToolButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QalculateToolButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19QalculateToolButtonE_t>.strings))
        return static_cast<void*>(this);
    return QToolButton::qt_metacast(_clname);
}

int QalculateToolButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QToolButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void QalculateToolButton::middleButtonClicked()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
